/*
 *  Copyright (C) 2015-2017  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "carne.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cmd.h"
#include "list.h"
#include "util.h"
#include "totais.h"

void carne_free(void *pointer)
{
	struct carne *carne = pointer;
	free(carne);
}

static struct carne * carne_new(char **args, int argc)
{
	struct carne *carne;
	int r = 0;
	carne = malloc(sizeof(*carne));
	r += set_int(args[1], &carne->mes);
	r += set_llong(args[2], &carne->rendimento);
	r += set_llong(args[3], &carne->alugueis);
	r += set_llong(args[4], &carne->outros);
	r += set_llong(args[5], &carne->exterior);
	r += set_llong(args[6], &carne->caixa);
	r += set_llong(args[7], &carne->alimentos);
	r += set_llong(args[8], &carne->dependentes);
	r += set_llong(args[9], &carne->previdencia);
	r += set_llong(args[10], &carne->base);
	r += set_llong(args[11], &carne->imposto);
	if (argc == 13) {
		r += set_int(args[12], &carne->dependente);
	} else {
		carne->dependente = 0;
	}
	if (r < 0 || carne->mes < 0 || carne->dependente < 0 ||
	    carne->rendimento < 0 || carne->alugueis < 0 ||
	    carne->outros < 0 || carne->exterior < 0 || carne->caixa < 0 ||
	    carne->alimentos < 0 || carne->dependentes < 0 ||
	    carne->previdencia < 0 || carne->base < 0 || carne->imposto < 0) {
		carne_free(carne);
		return NULL;
	}
	return carne;
}

static int run_carne(struct declaracao *dec, char **args, int argc)
{
	struct carne *carne;
	int r = 0;
	if (argc < 12 || argc > 13)
		return -EINVAL;
	carne = carne_new(args, argc);
	if (!carne)
		return -ENOMEM;
	if (carne->dependente > list_size(dec->dependentes)) {
		carne_free(carne);
		return -EINVAL;
	}
	r += totais_add(dec, "RENDTRIB", carne->rendimento);
	r += totais_add(dec, "RENDTRIB", carne->exterior);
	r += totais_add(dec, "RENDEXT", carne->exterior);
	r += totais_add(dec, "RENDPF", carne->rendimento);
	r += totais_add(dec, "PAGO", carne->imposto);
	r += totais_add(dec, "CARNE", carne->imposto);
	r += totais_add(dec, "INSS", carne->previdencia);
	if (carne->dependente) {
		r += totais_add(dec, "RENDPFDEP", carne->rendimento);
		r += totais_add(dec, "RENDEXTDEP", carne->exterior);
		r += totais_add(dec, "CARNEDEP", carne->imposto);
	} else {
		r += totais_add(dec, "RENDPFTIT", carne->rendimento);
		r += totais_add(dec, "RENDEXTTIT", carne->exterior);
		r += totais_add(dec, "CARNETIT", carne->imposto);
	}
	if (r) {
		carne_free(carne);
		return r;
	}
	r = list_add(&dec->carne, carne);
	if (r < 0) {
		carne_free(carne);
		return r;
	}
	return 0;
}

void carne_salva(struct declaracao *dec, FILE *f)
{
	int i;
	struct carne *j;
	for (i = 0; j = list_get(dec->carne, i); i++)
		fprintf(f, "carne %d %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %d\n",
			j->mes, j->rendimento, j->alugueis, j->outros,
			j->exterior, j->caixa, j->alimentos, j->dependentes,
			j->previdencia, j->base, j->imposto,
			j->dependente);
}

static struct cmd cmd_carne = {
	.name = "carne",
	.run = run_carne,
};

int carne_cmd_init(void)
{
	cmd_add(&cmd_carne);
	return 0;
}
